import tensorflow as tf
import pickle
import numpy as np
import keras
from keras.layers import Dense, Dropout, Input
from keras.optimizers import Adam
from keras.regularizers import l2
from keras.callbacks import ReduceLROnPlateau, ModelCheckpoint, EarlyStopping
import matplotlib.pyplot as plt
from keras import regularizers

val_size = 10000
pred_type = 'rs'

with open('parsed_data/deep_short6.parsed', 'rb') as fp:
    data = pickle.load(fp)

train_cols = ['x1', 'x2', 'y1', 'y2', 'h1', 'h2', 'rnorm', 'rnorm  max']
for i in range(128):
    if 'dofs{}'.format(i) not in train_cols:
        train_cols.append('dofs{}'.format(i))
        data['dofs{}'.format(i)] = 0.
target_cols = ['nref']

uniques=[None]
for i in range(5):
    print('orders{}'.format(i), data['orders{}'.format(i)].nunique())
    uniques.append(data['orders{}'.format(i)].nunique())
    if 'orders{}'.format(i) not in train_cols:
        target_cols.append('orders{}'.format(i))
    #     data['orders{}'.format(i)] = 0

    data['orders{}'.format(i)] = data['orders{}'.format(i)].astype('category').cat.codes


reduce_lr_callback = ReduceLROnPlateau(monitor='val_loss', factor=0.5,
                                       patience=5, min_lr=1e-12)
checkpoint_callback = ModelCheckpoint("checkpoint-hpada-" + pred_type, monitor='val_loss', verbose=0,
                                      save_best_only=True, save_weights_only=False, mode='auto', period=1)
early_stopping_callback = EarlyStopping(monitor='val_loss', min_delta=0, patience=100, verbose=1, mode='auto', baseline=None, restore_best_weights=True)


print(train_cols)
print(data.columns)
data['nref'] = data['nref'].astype('category').cat.codes
tmp = data['nref']
data = data.append([data[data['nref'] == 1]] * 50, ignore_index=True).sample(frac=1)
data = data.reindex(np.random.permutation(data.index))

print(data[data['nref'] == 1].shape[0] / data.shape[0])
train_data = data[train_cols]
target_data = data[target_cols]
print(target_data.shape)
print(train_data.shape)

val_data = data.sample(val_size)

val_data_target = val_data[target_cols]
val_data_train = val_data[train_cols]

# train_data=(train_data-train_data.min())/(train_data.max()-train_data.min())
l2_pen = 0.
dropout_rate = 0.
dropout_rate_head = 0.
epochs = 200
inputs = Input(shape=(136,))
nn = Dense(512, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(inputs)
nn = Dropout(dropout_rate_head)(nn)
nn = Dense(256, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
nn = Dropout(dropout_rate_head)(nn)
nn = Dense(256, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
nn = Dropout(dropout_rate_head)(nn)
nn = Dense(128, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
nn = Dropout(dropout_rate_head)(nn)
nn = Dense(128, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
nn = Dropout(dropout_rate_head)(nn)
nn = Dense(64, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
nn = Dropout(dropout_rate_head)(nn)
nn = Dense(64, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
nn = Dropout(dropout_rate_head)(nn)
nn = Dense(64, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
nn = Dropout(dropout_rate_head)(nn)
model_orders = []
for i in range(6):
    tmp = Dense(64, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
    tmp = Dropout(dropout_rate)(tmp)
    tmp = Dense(32, activation='relu', kernel_regularizer=regularizers.l2(l2_pen))(nn)
    tmp = Dropout(dropout_rate)(tmp)
    tmp = Dense(32, activation='relu')(tmp)
    tmp = Dropout(dropout_rate)(tmp)
    if i == 0:
        tmp = Dense(2, activation='softmax',name='nref')(tmp)
    else:
        tmp = Dense(uniques[i], activation='softmax', name='orders{}'.format(i-1))(tmp)
    model_orders.append(tmp)


total_model = keras.models.Model(inputs=inputs, outputs=model_orders)



adam = Adam(lr=0.001)

total_model.compile(loss='sparse_categorical_crossentropy', optimizer=adam, metrics=['acc'])

x = total_model.evaluate(x=val_data_train,
                         y=[val_data_target['nref'],
                            val_data_target['orders0'],
                            val_data_target['orders1'],
                            val_data_target['orders2'],
                            val_data_target['orders3'],
                            val_data_target['orders4']]
                         )
print(x)
h = total_model.fit(x=train_data, y=[target_data['nref'],
                                     target_data['orders0'],
                                     target_data['orders1'],
                                     target_data['orders2'],
                                     target_data['orders3'],
                                     target_data['orders4']],
                    validation_split=0.1,
                    verbose=1,
                    epochs=epochs,
                    batch_size=256,
                    callbacks=[reduce_lr_callback,
                               checkpoint_callback,
                               early_stopping_callback]
                    )
print(h.history.keys())
for k in h.history.keys():


    plt.plot(h.history[k], label=k)

    plt.legend()
    if 'acc' not in k: k="_"+k
    plt.savefig('plots/'+ k + ".svg")
    plt.close()
plt.figure()
for k in h.history.keys():
    if 'acc' in k and 'val' in k:
        plt.plot(h.history[k], label=k)

plt.legend()
plt.savefig("plots/common-val.svg")
plt.close()
plt.figure()
for k in h.history.keys():
    if 'acc' in k and 'val' not in k:
        plt.plot(h.history[k], label=k)
plt.legend()
plt.savefig("plots/common-train.svg")
